@@ -4,6 +4,7 @@
 # Copyright (c) 2015      - François D.
 # Copyright (c) 2017      - Mate Lakat
 # Copyright (c) 2018      - Edwin Steele
+# Copyright (c) 2021      - Tim AtLee
 
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to
@@ -32,20 +33,29 @@ import random
 import sys
 from collections import defaultdict
 from datetime import datetime
+from functools import cached_property
 from itertools import cycle
 from os.path import isfile, join, splitext
+from shutil import get_terminal_size
 from urllib.parse import quote as url_quote
 
-from click import get_terminal_size, progressbar
+from click import progressbar
 from natsort import natsort_keygen, ns
 from PIL import Image as PILImage
 
 from . import image, signals, video
 from .image import get_exif_tags, get_image_metadata, get_size, process_image
-from .settings import get_thumb
-from .utils import (Devnull, cached_property, check_or_create_dir, copy,
-                    get_mime, is_valid_html5_video, read_markdown,
-                    url_from_path)
+from .settings import Status, get_thumb
+from .utils import (
+    Devnull,
+    check_or_create_dir,
+    copy,
+    get_mime,
+    get_mod_date,
+    is_valid_html5_video,
+    read_markdown,
+    url_from_path,
+)
 from .video import process_video
 from .writer import AlbumListPageWriter, AlbumPageWriter
 
@@ -87,16 +97,10 @@ class Media:
 
         self.logger = logging.getLogger(__name__)
 
-        self.file_metadata = None
-        self._get_metadata()
-
-        # default: title is the filename
-        if not self.title:
-            self.title = self.basename
         signals.media_initialized.send(self)
 
     def __repr__(self):
-        return "<{}>({!r})".format(self.__class__.__name__, str(self))
+        return f"<{self.__class__.__name__}>({str(self)!r})"
 
     def __str__(self):
         return join(self.path, self.src_filename)
@@ -139,8 +143,12 @@ class Media:
             check_or_create_dir(orig_path)
             big_path = join(orig_path, self.src_filename)
             if not isfile(big_path):
-                copy(self.src_path, big_path, symlink=s['orig_link'],
-                     rellink=self.settings['rel_link'])
+                copy(
+                    self.src_path,
+                    big_path,
+                    symlink=s['orig_link'],
+                    rellink=self.settings['rel_link'],
+                )
             return join(s['orig_dir'], self.src_filename)
 
     @property
@@ -155,46 +163,67 @@ class Media:
 
         if not isfile(self.thumb_path):
             self.logger.debug('Generating thumbnail for %r', self)
-            path = (self.dst_path if os.path.exists(self.dst_path)
-                    else self.src_path)
+            path = self.dst_path if os.path.exists(self.dst_path) else self.src_path
             try:
                 # if thumbnail is missing (if settings['make_thumbs'] is False)
                 s = self.settings
                 if self.type == 'image':
                     image.generate_thumbnail(
-                        path, self.thumb_path, s['thumb_size'],
-                        fit=s['thumb_fit'])
+                        path, self.thumb_path, s['thumb_size'], fit=s['thumb_fit']
+                    )
                 elif self.type == 'video':
                     video.generate_thumbnail(
-                        path, self.thumb_path, s['thumb_size'],
-                        s['thumb_video_delay'], fit=s['thumb_fit'],
-                        converter=s['video_converter'])
+                        path,
+                        self.thumb_path,
+                        s['thumb_size'],
+                        s['thumb_video_delay'],
+                        fit=s['thumb_fit'],
+                        converter=s['video_converter'],
+                    )
             except Exception as e:
                 self.logger.error('Failed to generate thumbnail: %s', e)
                 return
         return url_from_path(self.thumb_name)
 
-    def _get_metadata(self):
-        """Get image metadata from filename.md: title, description, meta."""
-
-        self.description = ''
+    @cached_property
+    def description(self):
         """Description extracted from the Markdown <imagename>.md file."""
+        return self.markdown_metadata.get('description', '')
 
-        self.title = ''
-        """Title extracted from the Markdown <imagename>.md file."""
+    @cached_property
+    def title(self):
+        """Title extracted from the metadata, or defaults to the filename."""
+        title = self.markdown_metadata.get('title', '')
+        return title if title else self.basename
 
-        self.meta = {}
+    @cached_property
+    def meta(self):
         """Other metadata extracted from the Markdown <imagename>.md file."""
+        return self.markdown_metadata.get('meta', {})
 
-        descfile = splitext(self.src_path)[0] + '.md'
-        if isfile(descfile):
-            meta = read_markdown(descfile)
-            for key, val in meta.items():
-                setattr(self, key, val)
+    @cached_property
+    def markdown_metadata(self):
+        """Get metadata from filename.md: title, description, meta."""
+        return self._get_markdown_metadata()
+
+    @property
+    def markdown_metadata_filepath(self):
+        return splitext(self.src_path)[0] + '.md'
+
+    def _get_markdown_metadata(self):
+        """Get metadata from filename.md."""
+        meta = {'title': '', 'description': '', 'meta': {}}
+        if isfile(self.markdown_metadata_filepath):
+            meta.update(read_markdown(self.markdown_metadata_filepath))
+        return meta
+
+    @cached_property
+    def file_metadata(self):
+        """Type-specific metadata"""
+        return {}
 
     def _get_file_date(self):
-        stat = os.stat(self.src_path)
-        return datetime.fromtimestamp(stat.st_mtime)
+        return datetime.fromtimestamp(get_mod_date(self.src_path))
 
 
 class Image(Media):
@@ -217,8 +246,7 @@ class Image(Media):
     def date(self):
         """The date from the EXIF DateTimeOriginal metadata if available, or
         from the file date."""
-        return (self.exif and self.exif.get('dateobj', None) or
-                self._get_file_date())
+        return self.exif and self.exif.get('dateobj', None) or self._get_file_date()
 
     @cached_property
     def exif(self):
@@ -226,24 +254,29 @@ class Image(Media):
         information, see :ref:`simple-exif-data`.
         """
         datetime_format = self.settings['datetime_format']
-        return (get_exif_tags(self.raw_exif, datetime_format=datetime_format)
-                if self.raw_exif and self.src_ext in ('.jpg', '.jpeg') else None)
+        return (
+            get_exif_tags(self.raw_exif, datetime_format=datetime_format)
+            if self.raw_exif and self.src_ext in ('.jpg', '.jpeg')
+            else None
+        )
 
-    def _get_metadata(self):
-        super()._get_metadata()
-        self.file_metadata = get_image_metadata(self.src_path)
+    @cached_property
+    def file_metadata(self):
+        """Image file metadata (Exif and IPTC)"""
+        return get_image_metadata(self.src_path)
+
+    def _get_markdown_metadata(self):
+        """Get metadata from filename.md."""
+        meta = super()._get_markdown_metadata()
 
         # If a title or description hasn't been obtained by other means, look
         #  for the information in IPTC fields
-        if self.title and self.description:
-            # Nothing to do - we already have title and description
-            return
+        if not meta['title']:
+            meta['title'] = self.file_metadata['iptc'].get('title', '')
+        if not meta['description']:
+            meta['description'] = self.file_metadata['iptc'].get('description', '')
 
-        iptc_data = self.file_metadata['iptc']
-        if not self.title and iptc_data.get('title'):
-            self.title = iptc_data['title']
-        if not self.description and iptc_data.get('description'):
-            self.description = iptc_data['description']
+        return meta
 
     @cached_property
     def raw_exif(self):
@@ -278,7 +311,6 @@ class Video(Media):
 
     def __init__(self, filename, path, settings):
         super().__init__(filename, path, settings)
-        self.date = self._get_file_date()
 
         if not settings['use_orig'] or not is_valid_html5_video(self.src_ext):
             video_format = settings['video_format']
@@ -288,6 +320,22 @@ class Video(Media):
         else:
             self.mime = get_mime(self.src_ext)
 
+    @cached_property
+    def date(self):
+        """The date from the Date metadata if available, or from the file date."""
+        if 'date' in self.meta:
+            try:
+                self.logger.debug(
+                    "Reading date from image metadata : %s", self.src_filename
+                )
+                return datetime.fromisoformat(self.meta['date'][0])
+            except Exception:
+                self.logger.debug(
+                    "Reading date from image metadata failed : %s", self.src_filename
+                )
+        # If no date is found in the metadata, return the file date.
+        return self._get_file_date()
+
 
 class Album:
     """Gather all informations on an album.
@@ -325,13 +373,15 @@ class Album:
             self.dst_path = join(settings['destination'], path)
 
         self.logger = logging.getLogger(__name__)
-        self._get_metadata()
 
         # optionally add index.html to the URLs
         self.url_ext = self.output_file if settings['index_in_url'] else ''
 
-        self.index_url = url_from_path(os.path.relpath(
-            settings['destination'], self.dst_path)) + '/' + self.url_ext
+        self.index_url = (
+            url_from_path(os.path.relpath(settings['destination'], self.dst_path))
+            + '/'
+            + self.url_ext
+        )
 
         #: List of all medias in the album (:class:`~sigal.gallery.Image` and
         #: :class:`~sigal.gallery.Video`).
@@ -340,26 +390,34 @@ class Album:
 
         for f in filenames:
             ext = splitext(f)[1]
+            media = None
             if ext.lower() in settings['img_extensions']:
                 media = Image(f, self.path, settings)
             elif ext.lower() in settings['video_extensions']:
                 media = Video(f, self.path, settings)
-            else:
-                continue
 
-            self.medias_count[media.type] += 1
-            medias.append(media)
+            # Allow modification of the media, including overriding the class
+            # type for the media.
+            result = signals.album_file.send(self, filename=f, media=media)
+            for recv, ret in result:
+                if ret is not None:
+                    media = ret
+
+            if media:
+                self.medias_count[media.type] += 1
+                medias.append(media)
 
         signals.album_initialized.send(self)
 
     def __repr__(self):
         return "<{}>(path={!r}, title={!r})".format(
-            self.__class__.__name__, self.path, self.title)
+            self.__class__.__name__, self.path, self.title
+        )
 
     def __str__(self):
-        return (f'{self.path} : ' +
-                ', '.join(f'{count} {_type}s'
-                          for _type, count in self.medias_count.items()))
+        return f'{self.path} : ' + ', '.join(
+            f'{count} {_type}s' for _type, count in self.medias_count.items()
+        )
 
     def __len__(self):
         return len(self.medias)
@@ -367,36 +425,49 @@ class Album:
     def __iter__(self):
         return iter(self.medias)
 
-    def _get_metadata(self):
-        """Get album metadata from `description_file` (`index.md`):
+    @cached_property
+    def description(self):
+        """Description extracted from the Markdown index.md file."""
+        return self.markdown_metadata.get('description', '')
 
-        -> title, thumbnail image, description
+    @cached_property
+    def title(self):
+        """Title extracted from the Markdown index.md file."""
+        title = self.markdown_metadata.get('title', '')
+        path = self.path if self.path != '.' else self.src_path
+        return title if title else os.path.basename(path)
 
-        """
-        descfile = join(self.src_path, self.description_file)
-        self.description = ''
-        self.meta = {}
-        # default: get title from directory name
-        self.title = os.path.basename(self.path if self.path != '.'
-                                      else self.src_path)
-
-        if isfile(descfile):
-            meta = read_markdown(descfile)
-            for key, val in meta.items():
-                setattr(self, key, val)
+    @cached_property
+    def meta(self):
+        """Other metadata extracted from the Markdown index.md file."""
+        return self.markdown_metadata.get('meta', {})
 
+    @cached_property
+    def author(self):
+        """Author extracted from the Markdown index.md file or settings."""
         try:
-            self.author = self.meta['author'][0]
+            return self.meta['author'][0]
         except KeyError:
-            self.author = self.settings.get('author')
+            return self.settings.get('author')
+
+    @property
+    def markdown_metadata_filepath(self):
+        return join(self.src_path, self.description_file)
+
+    @cached_property
+    def markdown_metadata(self):
+        """Get metadata from filename.md: title, description, meta."""
+        meta = {'title': '', 'description': '', 'meta': {}}
+        if isfile(self.markdown_metadata_filepath):
+            meta.update(read_markdown(self.markdown_metadata_filepath))
+        return meta
 
     def create_output_directories(self):
         """Create output directories for thumbnails and original images."""
         check_or_create_dir(self.dst_path)
 
         if self.medias:
-            check_or_create_dir(join(self.dst_path,
-                                     self.settings['thumb_dir']))
+            check_or_create_dir(join(self.dst_path, self.settings['thumb_dir']))
 
         if self.medias and self.settings['keep_orig']:
             self.orig_path = join(self.dst_path, self.settings['orig_dir'])
@@ -425,6 +496,7 @@ class Album:
                     return album.meta.get(meta_key, [''])[0]
 
             else:
+
                 def sort_key(s):
                     album = self.gallery.albums[join(root_path, s)]
                     return getattr(album, albums_sort_attr)
@@ -444,15 +516,14 @@ class Album:
             elif medias_sort_attr.startswith('meta.'):
                 meta_key = medias_sort_attr.split(".", 1)[1]
                 key = natsort_keygen(
-                    key=lambda s: s.meta.get(meta_key, [''])[0],
-                    alg=ns.LOCALE)
+                    key=lambda s: s.meta.get(meta_key, [''])[0], alg=ns.LOCALE
+                )
             else:
                 key = natsort_keygen(
-                    key=lambda s: getattr(s, medias_sort_attr),
-                    alg=ns.LOCALE)
+                    key=lambda s: getattr(s, medias_sort_attr), alg=ns.LOCALE
+                )
 
-            self.medias.sort(key=key,
-                             reverse=self.settings['medias_sort_reverse'])
+            self.medias.sort(key=key, reverse=self.settings['medias_sort_reverse'])
 
         signals.medias_sorted.send(self)
 
@@ -476,8 +547,7 @@ class Album:
         sub-directory.
         """
         root_path = self.path if self.path != '.' else ''
-        return [self.gallery.albums[join(root_path, path)]
-                for path in self.subdirs]
+        return [self.gallery.albums[join(root_path, path)] for path in self.subdirs]
 
     @property
     def nbmedias(self):
@@ -501,8 +571,9 @@ class Album:
         thumbnail = self.meta.get('thumbnail', [''])[0]
 
         if thumbnail and isfile(join(self.src_path, thumbnail)):
-            self._thumbnail = url_from_path(join(
-                self.name, get_thumb(self.settings, thumbnail)))
+            self._thumbnail = url_from_path(
+                join(self.name, get_thumb(self.settings, thumbnail))
+            )
             self.logger.debug("Thumbnail for %r : %s", self, self._thumbnail)
             return self._thumbnail
         else:
@@ -520,15 +591,17 @@ class Album:
 
                 if size['width'] > size['height']:
                     try:
-                        self._thumbnail = (url_quote(self.name) + '/' +
-                                           f.thumbnail)
+                        self._thumbnail = url_quote(self.name) + '/' + f.thumbnail
                     except Exception as e:
-                        self.logger.info("Failed to get thumbnail for %s: %s",
-                                         f.dst_filename, e)
+                        self.logger.info(
+                            "Failed to get thumbnail for %s: %s", f.dst_filename, e
+                        )
                     else:
                         self.logger.debug(
                             "Use 1st landscape image as thumbnail for %r : %s",
-                            self, self._thumbnail)
+                            self,
+                            self._thumbnail,
+                        )
                         return self._thumbnail
 
             # else simply return the 1st media file
@@ -536,12 +609,14 @@ class Album:
                 for media in self.medias:
                     if media.thumbnail is not None:
                         try:
-                            self._thumbnail = (url_quote(self.name) + '/' +
-                                               media.thumbnail)
+                            self._thumbnail = (
+                                url_quote(self.name) + '/' + media.thumbnail
+                            )
                         except Exception as e:
                             self.logger.info(
                                 "Failed to get thumbnail for %s: %s",
-                                media.dst_filename, e
+                                media.dst_filename,
+                                e,
                             )
                         else:
                             break
@@ -549,19 +624,21 @@ class Album:
                     self.logger.warning("No thumbnail found for %r", self)
                     return
 
-                self.logger.debug("Use the 1st image as thumbnail for %r : %s",
-                                  self, self._thumbnail)
+                self.logger.debug(
+                    "Use the 1st image as thumbnail for %r : %s", self, self._thumbnail
+                )
                 return self._thumbnail
 
             # use the thumbnail of their sub-directories
             if not self._thumbnail:
                 for path, album in self.gallery.get_albums(self.path):
                     if album.thumbnail:
-                        self._thumbnail = (url_quote(self.name) + '/' +
-                                           album.thumbnail)
+                        self._thumbnail = url_quote(self.name) + '/' + album.thumbnail
                         self.logger.debug(
                             "Using thumbnail from sub-directory for %r : %s",
-                            self, self._thumbnail)
+                            self,
+                            self._thumbnail,
+                        )
                         return self._thumbnail
 
         self.logger.error('Thumbnail not found for %r', self)
@@ -569,8 +646,7 @@ class Album:
     @property
     def random_thumbnail(self):
         try:
-            return url_from_path(join(self.name,
-                                      random.choice(self.medias).thumbnail))
+            return url_from_path(join(self.name, random.choice(self.medias).thumbnail))
         except IndexError:
             return self.thumbnail
 
@@ -590,8 +666,7 @@ class Album:
             if path == '.':
                 break
 
-            url = (url_from_path(os.path.relpath(path, self.path)) + '/' +
-                   self.url_ext)
+            url = url_from_path(os.path.relpath(path, self.path)) + '/' + self.url_ext
             breadcrumb.append((url, self.gallery.albums[path].title))
 
         breadcrumb.reverse()
@@ -599,8 +674,7 @@ class Album:
 
     @property
     def show_map(self):
-        """Check if we have at least one photo with GPS location in the album
-        """
+        """Check if we have at least one photo with GPS location in the album"""
         return any(image.has_location() for image in self.images)
 
     @cached_property
@@ -611,7 +685,6 @@ class Album:
 
 
 class Gallery:
-
     def __init__(self, settings, ncpu=None, quiet=False):
         self.settings = settings
         self.logger = logging.getLogger(__name__)
@@ -619,6 +692,9 @@ class Gallery:
         self.init_pool(ncpu)
         check_or_create_dir(settings['destination'])
 
+        if settings['max_img_pixels']:
+            PILImage.MAX_IMAGE_PIXELS = settings['max_img_pixels']
+
         # Build the list of directories with images
         albums = self.albums = {}
         src_path = self.settings['source']
@@ -627,20 +703,22 @@ class Gallery:
         ignore_files = settings['ignore_files']
 
         progressChars = cycle(["/", "-", "\\", "|"])
-        show_progress = (not quiet and
-                         self.logger.getEffectiveLevel() >= logging.WARNING and
-                         os.isatty(sys.stdout.fileno()))
+        show_progress = (
+            not quiet
+            and self.logger.getEffectiveLevel() >= logging.WARNING
+            and os.isatty(sys.stdout.fileno())
+        )
         self.progressbar_target = None if show_progress else Devnull()
 
-        for path, dirs, files in os.walk(src_path, followlinks=True,
-                                         topdown=False):
+        for path, dirs, files in os.walk(src_path, followlinks=True, topdown=False):
             if show_progress:
                 print("\rCollecting albums " + next(progressChars), end="")
             relpath = os.path.relpath(path, src_path)
 
             # Test if the directory match the ignore_dirs settings
-            if ignore_dirs and any(fnmatch.fnmatch(relpath, ignore)
-                                   for ignore in ignore_dirs):
+            if ignore_dirs and any(
+                fnmatch.fnmatch(relpath, ignore) for ignore in ignore_dirs
+            ):
                 self.logger.info('Ignoring %s', relpath)
                 continue
 
@@ -673,13 +751,19 @@ class Gallery:
         if show_progress:
             print("\rCollecting albums, done.")
 
-        with progressbar(albums.values(), label="%16s" % "Sorting albums",
-                         file=self.progressbar_target) as progress_albums:
+        with progressbar(
+            albums.values(),
+            label="%16s" % "Sorting albums",
+            file=self.progressbar_target,
+        ) as progress_albums:
             for album in progress_albums:
                 album.sort_subdirs(settings['albums_sort_attr'])
 
-        with progressbar(albums.values(), label="%16s" % "Sorting media",
-                         file=self.progressbar_target) as progress_albums:
+        with progressbar(
+            albums.values(),
+            label="%16s" % "Sorting media",
+            file=self.progressbar_target,
+        ) as progress_albums:
             for album in progress_albums:
                 album.sort_medias(settings['medias_sort_attr'])
 
@@ -708,7 +792,12 @@ class Gallery:
 
         self.logger.info("Using %s cores", ncpu)
         if ncpu > 1:
-            self.pool = multiprocessing.Pool(processes=ncpu)
+
+            self.pool = multiprocessing.Pool(
+                processes=ncpu,
+                initializer=pool_init,
+                initargs=(self.settings['max_img_pixels'],),
+            )
         else:
             self.pool = None
 
@@ -737,17 +826,24 @@ class Gallery:
                 return ""
 
         try:
-            with progressbar(self.albums.values(), label="Collecting files",
-                             item_show_func=log_func, show_eta=False,
-                             file=self.progressbar_target) as albums:
-                media_list = [f for album in albums
-                              for f in self.process_dir(album, force=force)]
+            with progressbar(
+                self.albums.values(),
+                label="Collecting files",
+                item_show_func=log_func,
+                show_eta=False,
+                file=self.progressbar_target,
+            ) as albums:
+                media_list = [
+                    f for album in albums for f in self.process_dir(album, force=force)
+                ]
         except KeyboardInterrupt:
             sys.exit('Interrupted')
 
-        bar_opt = {'label': "Processing files",
-                   'show_pos': True,
-                   'file': self.progressbar_target}
+        bar_opt = {
+            'label': "Processing files",
+            'show_pos': True,
+            'file': self.progressbar_target,
+        }
 
         if self.pool:
             result = []
@@ -756,8 +852,6 @@ class Gallery:
                     for status in self.pool.imap_unordered(worker, media_list):
                         result.append(status)
                         bar.update(1)
-                self.pool.close()
-                self.pool.join()
             except KeyboardInterrupt:
                 self.pool.terminate()
                 sys.exit('Interrupted')
@@ -766,26 +860,34 @@ class Gallery:
                     "Failed to process files with the multiprocessing feature."
                     " This can be caused by some module import or object "
                     "defined in the settings file, which can't be serialized.",
-                    exc_info=True)
+                    exc_info=True,
+                )
                 sys.exit('Abort')
+            finally:
+                self.pool.close()
+                self.pool.join()
         else:
             with progressbar(media_list, **bar_opt) as medias:
                 result = [process_file(media_item) for media_item in medias]
 
         if any(result):
-            failed_files = [media for status, media in zip(result, media_list)
-                            if status != 0]
+            failed_files = [
+                media for status, media in zip(result, media_list) if status != 0
+            ]
             self.remove_files(failed_files)
 
         if self.settings['write_html']:
-            album_writer = AlbumPageWriter(self.settings,
-                                           index_title=self.title)
-            album_list_writer = AlbumListPageWriter(self.settings,
-                                                    index_title=self.title)
-            with progressbar(self.albums.values(),
-                             label="%16s" % "Writing files",
-                             item_show_func=log_func, show_eta=False,
-                             file=self.progressbar_target) as albums:
+            album_writer = AlbumPageWriter(self.settings, index_title=self.title)
+            album_list_writer = AlbumListPageWriter(
+                self.settings, index_title=self.title
+            )
+            with progressbar(
+                self.albums.values(),
+                label="%16s" % "Writing files",
+                item_show_func=log_func,
+                show_eta=False,
+                file=self.progressbar_target,
+            ) as albums:
                 for album in albums:
                     if album.albums:
                         if album.medias:
@@ -793,7 +895,8 @@ class Gallery:
                                 "Album %s contains sub-albums and images. "
                                 "Please move images to their own sub-album. "
                                 "Images in album %s will not be visible.",
-                                album.title, album.title
+                                album.title,
+                                album.title,
                             )
                         album_list_writer.write(album)
                     else:
@@ -812,8 +915,10 @@ class Gallery:
                     self.stats[f.type + '_failed'] += 1
                     album.medias.remove(f)
                     break
-        self.logger.error('You can run "sigal build" in verbose (--verbose) or'
-                          ' debug (--debug) mode to get more details.')
+        self.logger.error(
+            'You can run "sigal build" in verbose (--verbose) or'
+            ' debug (--debug) mode to get more details.'
+        )
 
     def process_dir(self, album, force=False):
         """Process a list of images in a directory."""
@@ -826,9 +931,29 @@ class Gallery:
                 yield f
 
 
+def pool_init(max_img_pixels):
+    if max_img_pixels:
+        PILImage.MAX_IMAGE_PIXELS = max_img_pixels
+
+
 def process_file(media):
-    processor = process_image if media.type == 'image' else process_video
-    return processor(media)
+    processor = None
+    if media.type == 'image':
+        processor = process_image
+    elif media.type == 'video':
+        processor = process_video
+
+    # Allow overriding of the processor
+    result = signals.process_file.send(media, processor=processor)
+    for recv, ret in result:
+        if ret is not None:
+            processor = ret
+
+    if processor:
+        return processor(media)
+    else:
+        logging.warning('Processor not found for media %s', media.path)
+        return Status.FAILURE
 
 
 def worker(args):
