@@ -9,3 +9,5 @@ media_initialized = signal('media_initialized')
 albums_sorted = signal('albums_sorted')
 medias_sorted = signal('medias_sorted')
 before_render = signal('before_render')
+album_file = signal('album_file')
+process_file = signal('process_file')
